// RTSP Server

#include "depthai/depthai.hpp"
#include "xop/RtspServer.h"
#include "net/Timer.h"
#include <thread>
#include <memory>
#include <iostream>
#include <string>


void SendFrameThread(xop::RtspServer *rtsp_server, xop::MediaSessionId session_id,
                     const std::shared_ptr<dai::DataOutputQueue>& outqueue);

int main(int argc, char **argv) {
    dai::Pipeline pipeline;
    //定义
    auto colorCamera = pipeline.create<dai::node::ColorCamera>();
    colorCamera->setBoardSocket(dai::CameraBoardSocket::RGB);
    colorCamera->setResolution(dai::ColorCameraProperties::SensorResolution::THE_1080_P);
//    colorCamera->setIspScale(1,3);
    colorCamera->setFps(20);

//    colorCamera->setVideoSize(1920, 1080);
    auto videoEncoder = pipeline.create<dai::node::VideoEncoder>();
    videoEncoder->setDefaultProfilePreset(colorCamera->getFps(),
                                          dai::VideoEncoderProperties::Profile::H264_HIGH);


    colorCamera->video.link(videoEncoder->input);

    //定义输出
    auto xLinkOut = pipeline.create<dai::node::XLinkOut>();
    xLinkOut->setStreamName("out");

    videoEncoder->bitstream.link(xLinkOut->input);

    //结构推送相机
    dai::Device device(pipeline);
    //取帧显示
    auto outputQueue = device.getOutputQueue("out", int(colorCamera->getFps()), false);//maxsize 代表缓冲数据


    std::string suffix = "live";
    std::string ip = "127.0.0.1";
    std::string port = "5554";
    std::string rtsp_url = "rtsp://" + ip + ":" + port + "/" + suffix;

    std::shared_ptr<xop::EventLoop> event_loop(new xop::EventLoop());
    std::shared_ptr<xop::RtspServer> server = xop::RtspServer::Create(event_loop.get());

    if (!server->Start("0.0.0.0", atoi(port.c_str()))) {
        printf("RTSP Server listen on %s failed.\n", port.c_str());
        return 0;
    }

#ifdef AUTH_CONFIG
    server->SetAuthConfig("-_-", "admin", "12345");
#endif

    xop::MediaSession *session = xop::MediaSession::CreateNew("live");
    session->AddSource(xop::channel_0, xop::H264Source::CreateNew());
    //session->StartMulticast();
    session->AddNotifyConnectedCallback([](xop::MediaSessionId sessionId, const std::string& peer_ip, uint16_t peer_port) {
        printf("RTSP client connect, ip=%s, port=%hu \n", peer_ip.c_str(), peer_port);
    });

    session->AddNotifyDisconnectedCallback([](xop::MediaSessionId sessionId, const std::string& peer_ip, uint16_t peer_port) {
        printf("RTSP client disconnect, ip=%s, port=%hu \n", peer_ip.c_str(), peer_port);
    });

    xop::MediaSessionId session_id = server->AddSession(session);

    std::thread t1(SendFrameThread, server.get(), session_id, outputQueue);
    t1.detach();

    std::cout << "Play URL: " << rtsp_url << std::endl;

    while (not device.isClosed()) {
        xop::Timer::Sleep(100);
    }

    return 0;
}

void SendFrameThread(xop::RtspServer *rtsp_server, xop::MediaSessionId session_id,
                     const std::shared_ptr<dai::DataOutputQueue>& outqueue) {

    while (true) {
        auto h264Packet = outqueue->get<dai::ImgFrame>();

        auto buffer = h264Packet->getData();

        if (not buffer.empty()) {
            xop::AVFrame videoFrame = {0};
            videoFrame.type = 0;
            videoFrame.size = buffer.size();
            videoFrame.timestamp = xop::H264Source::GetTimestamp();
            videoFrame.buffer.reset(new uint8_t[videoFrame.size]);
            memcpy(videoFrame.buffer.get(), (uint8_t *) buffer.data(), videoFrame.size);
            rtsp_server->PushFrame(session_id, xop::channel_0, videoFrame);
        } else {
            break;
        }
    };
}

